<?php

namespace Luticate\Auth\Controller;

use Luticate\Auth\DBO\LuticateUsersDbo;
use Luticate\Utils\LuController;
use Luticate\Auth\Business\LuticateSettingsUsersBusiness;
use Luticate\Auth\DBO\LuticateSettingsUsersDbo;

class LuticateSettingsUsersController extends LuController {
    protected function getBusiness()
    {
        return new LuticateSettingsUsersBusiness();
    }

    /**
     * Get all settings for a user
     * @param $user_id int The user id
     * @param int $page The page number, 0 based
     * @param int $perPage The number of items per page
     * @return \Luticate\Utils\LuMultipleDbo
     */
    public function getAllByUser($user_id, $page = 0, $perPage = 2000000000)
    {
        return LuticateSettingsUsersBusiness::getAllByUser($user_id, $page, $perPage);
    }

    /**
     * Get a setting for a user
     * @param $user_id int The user id
     * @param $setting_name string The setting name
     * @return \Luticate\Utils\LuDbo
     */
    public function get($user_id, $setting_name)
    {
        return LuticateSettingsUsersBusiness::checkSettingUserName($user_id, $setting_name);
    }

    /**
     * Edit a setting for a user
     * @param $user_id int The user id
     * @param $setting_name string The setting name
     * @param $setting LuticateSettingsUsersDbo The setting
     * @return bool
     */
    public function edit($user_id, $setting_name, LuticateSettingsUsersDbo $setting)
    {
        $setting->setUserId($user_id);
        $setting->setName($setting_name);
        return LuticateSettingsUsersBusiness::edit($setting, true);
    }

    /**
     * Edit a setting for the logged user
     * @param $_user LuticateUsersDbo The logged user
     * @param $setting_name string The setting name
     * @param $setting_value string The setting value
     * @return bool
     */
    public function editMe($_user, $setting_name, $setting_value)
    {
        $setting = new LuticateSettingsUsersDbo();
        $setting->setUserId($_user->getId());
        $setting->setName($setting_name);
        $setting->setIsBlocked(null);
        $setting->setValue(json_decode($setting_value));
        return LuticateSettingsUsersBusiness::edit($setting, false);
    }
}