<?php

namespace Luticate\Auth\DataAccess;

use Luticate\Auth\DataAccess\Models\LuticateUsers;
use Luticate\Utils\LuDataAccess;
use Luticate\Auth\DataAccess\Models\LuticateSettingsUsers;
use Luticate\Auth\DBO\LuticateSettingsUsersDbo;

class LuticateSettingsUsersDataAccess extends LuDataAccess {
    protected static function getModel()
    {
        return new LuticateSettingsUsers();
    }

    public static function getModelByName($user_id, $setting_name)
    {
        return LuticateSettingsUsers::where("name", "=", $setting_name)->where("user_id", "=", $user_id)->first();
    }

    public static function getAllByUser($user_id, $page = 0, $perPage = 2000000000)
    {
        return self::getMultiple(array(array('user_id', '=', $user_id)), array(array("name", "ASC")), $page, $perPage);
    }

    /**
     * @param $user_id int
     * @param $setting_name string
     * @return LuticateSettingsUsersDbo
     */
    public static function getByName($user_id, $setting_name)
    {
        $setting = self::getModelByName($user_id, $setting_name);
        if (is_null($setting)) {
            return null;
        }
        return $setting->toDbo();
    }

    public static function delete($user_id, $setting_name)
    {
        LuticateUsers::find($user_id)->settings()->detach($setting_name);
    }

    /**
     * @param $user_setting LuticateSettingsUsersDbo
     */
    public static function edit($user_setting)
    {
        $user_setting_model = self::getModelByName($user_setting->getUserId(), $user_setting->getName());
        if (is_null($user_setting))
            return;
        $user_setting_model->value = $user_setting->getValue();
        $user_setting_model->is_blocked = $user_setting->getIsBlocked();
        $user_setting_model->save();
    }
}